/*
 * $Id: lab-timer.c,v 1.2 1999/03/05 13:28:23 bunka Exp $
 *
 *   Lab3Timer
 *   A small timer application working like these famous Tripple
 *   Laboratory timers.
 *
 *   Copyright (C) 1999, Sebastian Bunka <Sebastian.Bunka@vu-wien.ac.at>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; see the file COPYING.  If not, write to
 *   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 *
 *
 * first form; i have the following objects defined in the form
 * FL_FORM *layout
 *
 * FL_OBJECT
 *        *background,
 *        *timer_field,  # background for the timerfield
 *        *t_quit,       # quit program
 *        *t_clear,      # clear selected timer field
 *        *t_stop,       # suspend selected timer
 *        *t_start,      # start selected/default timer
 *	  *t_logo,    	 # the logo at the top of the timer
 *	  *t_tooltip,
 *        *t_hour,
 *        *t_minute10,
 *        *t_minute,
 *        *t_timer[3],   # array of the three timers
 *        *t_button[3],  # array of timer select button
 *        *t_comment[3]; # array of the three commant fields
 * 
 * second form
 *
 *	FL_FORM *help_window; 	# The help window
 *
 *	FL_OBJECT
 *		*help_browser,	# object to write the help lines to
 *		*help_hide;	# object to hide the help window
 *
 *
 * third form
 *
 *	FL_FORM *msg_window;	# message window for alarm
 *
 *	FL_OBJECT
 *		*msg_text1,	# text header for alarm
 *		*t_msg_hide,	# object to hide msg window
 *		*msg_text2,	# comment header
 *		*msg_text3;	# the comment string one has entered
 */ 


#include "forms.h"
#include "layout.h"
#include "msg_window.h"
#include "help_window.h"
#include "my_config.h"
#include "lab-timer.h"
#include <unistd.h>
#include <stdlib.h>



/*
   ##################################################
   the main 
   ##################################################
*/

int main(int argc, char *argv[])
{

  FL_OBJECT *obj;
  float t_collect_secs[3] = {0,0,0};     	/* collect the seconds */
  int i;
  char *env;
  extern char *getenv();



#ifdef TESTMODE
  int test_mode=1, test_value=TESTTIME; 
#endif



   fl_initialize(&argc, argv, "Lab3Timer", 0, 0);


/* now get app-resources defined outside of main above */

   fl_get_app_resources(res,7);


if (run_prog[0]) {
	strcat(&run_prog[0], " &");
};


/* check if an env variable is set for the program to run 
 * overrides the definition in app-resources , but it is overridden
 * by the cmd-line!
 */
   
  if ((env = getenv("L3TPROG")) != NULL) {
        strcpy(&run_prog[0], env);
        strcat(&run_prog[0], " &");
  }


  get_cmdl(argc,argv);

  
/* create, but do NOT show the three forms */

   create_form_layout();
   create_form_msg_window();
   create_form_help_window();
   
/*   fl_adjust_form_size(layout);  needs too much work */


/* ************ initialization *************************
 * default timer, logo pixmap, fonts, other stuff
 */


 
/*  define timer display format by function timer_filter */
/*  switch off scroll bars for input fields */

   for (i=0;i<3;i++) {
	if (tiny_mode == 1) {
		fl_set_object_lsize(t_timer[i], FL_NORMAL_SIZE);
		fl_set_object_lsize(t_comment[i] , FL_SMALL_SIZE);
	}
     fl_set_timer_filter(t_timer[i],timer_filter);
     fl_set_input_hscrollbar(t_comment[i],FL_OFF);
     fl_set_input_vscrollbar(t_comment[i],FL_OFF);

   }

/*  switch on the first(default) timer button */

	t_select = 0;	/* set the default timer */
	fl_set_button(t_button[t_select],1);

/* set the logo */

	fl_free_pixmap_pixmap(t_logo);
	if (tiny_mode == 1) {
		fl_set_pixmap_data(t_logo,LOGOSMALL);
	} else {
		fl_set_pixmap_data(t_logo,LOGONORMAL);
	}

	
/* get the icon pixmap */

	t_icon1 = fl_create_from_pixmapdata(fl_root, l3t_normal_xpm, &w, &h, &mask1, 0, 0, 0);
	t_icon2 = fl_create_from_pixmapdata(fl_root, l3t_alarm_xpm, &w, &h, &mask2, 0, 0, 0);

/* set the standard iconized pixmap */

	fl_set_form_icon(layout, t_icon1, mask1);

/* set the posthandler for the timers, when alarm/mouse enteres/push */

	fl_set_object_posthandler(t_timer[0], post_timer);
	fl_set_object_posthandler(t_timer[1], post_timer);
	fl_set_object_posthandler(t_timer[2], post_timer);

/* set the help tips for the buttons */

	set_tip(t_quit, "Want to quit ?\nPress me");
	set_tip(t_start,"Start/Resume the\nselected Timer");
	set_tip(t_stop,"Suspend the\nselected Timer");
	set_tip(t_clear,"Clear the\nselected Timer\nand Comment");
	set_tip(t_logo,ABOUT);
	set_tip(t_help,"Open the Help Window");
	set_tip(t_tooltip,"Switch on/off tool tips");
	set_tip(t_hour,"1 Hour Increase\nof selected Timer");
	set_tip(t_minute10,"10 Min. Increase\nof selected Timer");
	set_tip(t_minute,"1 Min. Increase\nof selected Timer");

	for (i=0;i<3;i++) {
	   set_tip(t_button[i],"Select this timer for\nSet/Start/Stop/Clear");
	   set_tip(t_comment[i],"Enter your Comment/\nKeyword for this Timer");
	}
	

	if (no_tips == 0) {
		fl_set_button(t_tooltip,1);
	} else {
		fl_set_button(t_tooltip,0);
	}


/* Other redefinitions for a tiny timer */

if (tiny_mode == 1) {
	fl_set_object_lsize(t_start , FL_SMALL_SIZE);
	fl_set_object_lsize(t_stop , FL_SMALL_SIZE);
	fl_set_object_lsize(t_clear , FL_SMALL_SIZE);
	fl_set_object_lsize(t_quit , FL_TINY_SIZE);
	fl_set_object_lsize(t_hour , FL_TINY_SIZE);
	fl_set_object_lsize(t_minute , FL_TINY_SIZE);
	fl_set_object_lsize(t_minute10 , FL_TINY_SIZE);
	fl_set_object_lsize(t_help , FL_NORMAL_SIZE);
	
	fl_scale_form(layout,0.7,0.7);
}

	
/* ************ end of init ************************* */


/* show the first (main) form */

   fl_set_form_position(layout,x_pos,y_pos);
   fl_show_form(layout,FL_PLACE_GEOMETRY,FL_FULLBORDER,"Lab3Timer");


/* ********** the button actions / main loop ***************************** */

  while((obj = fl_do_forms()) ) /* the main loop, waiting for actions or quit */
  {

	if (obj == t_button[0]) {
		t_select = 0;
		continue;

	} else if (obj == t_button[1]) {
		t_select = 1;
		continue;

	} else if (obj == t_button[2]) {
		t_select = 2;
		continue;

	} else if (obj == t_logo) {
   			fl_show_oneliner(ABOUT, obj->form->x + obj->x,
                      	obj->form->y + obj->y + obj->h + 1);
			continue;

	} else if (obj == t_tooltip) {
		if (no_tips == 0) {
			no_tips = 1;
		} else {
			no_tips = 0;
		}
		;



	} else if (obj == t_start) {
/* start button */
		if ( ! fl_get_timer(t_timer[t_select]) ) {
#ifdef TESTMODE
			if (test_mode == 1) {
				fl_set_timer(t_timer[t_select],test_value);
				t_is_running[t_select] = 1;
				t_stopped[t_select] = 0;
			}
#endif
			continue;
		} else if (t_stopped[t_select] == 1) {
			fl_resume_timer(t_timer[t_select]);
			t_is_running[t_select] = 1;
			t_stopped[t_select] = 0;
		} 
		continue; 


	} else if (obj == t_stop) {
/* stop button */
		if (t_is_running[t_select] == 1 && timer_blinking[t_select] == 0) { 
			fl_suspend_timer(t_timer[t_select]);
			t_stopped[t_select] = 1; 
			t_is_running[t_select] = 0;
		}  
		continue;


	} else if (obj == t_clear) {
/* clear selected timer completely
 * AND reset the alarm icon to normal icon */
		fl_set_timer(t_timer[t_select],0.0);
		fl_set_input(t_comment[t_select],"");
		timer_blinking[t_select] = 0;
		check_blink(0);
		t_collect_secs[t_select] = 0;
		t_is_running[t_select] = 0;


	} else if (obj == t_msg_hide) {
/* hide a msg_window */
		fl_hide_form(msg_window);


	} else if (obj == help_hide) {
/* hide the help window */
		fl_hide_form(help_window);


	} else if (obj == t_help) {
/* open help window */
		show_help_window();


	} else if (obj == t_hour && t_is_running[t_select] == 0) {
/* the hour button */
		t_collect_secs[t_select] = fl_get_timer(t_timer[t_select]);
#ifdef LINUX_CONSOLE                 /* have a nice beep when pressing a button ? */
		if (linux_beep == 1) alarm_linux(4700,40);
#endif
		if (t_collect_secs[t_select] < MAXTIME) {	/* MAXTIME possible -> my_config.h */
			t_collect_secs[t_select] += 3600;	/* each press +3600 secs */
		}
		fl_set_timer(t_timer[t_select],t_collect_secs[t_select]);
		fl_suspend_timer(t_timer[t_select]);	/* set timer, and suspend immediately */
		t_stopped[t_select] = 1;		/* signal timaer's not running */



	} else if (obj == t_minute10 && t_is_running[t_select] == 0) {
/* 10 min button */
		t_collect_secs[t_select] = fl_get_timer(t_timer[t_select]);
#ifdef LINUX_CONSOLE
		if (linux_beep == 1) alarm_linux(4700,40);
#endif
		if (t_collect_secs[t_select] < MAXTIME) {
			t_collect_secs[t_select] += 600;	/* 10 min = 600 secs */
		}
		fl_set_timer(t_timer[t_select],t_collect_secs[t_select]);
		fl_suspend_timer(t_timer[t_select]);
		t_stopped[t_select] = 1;



	} else if (obj == t_minute && t_is_running[t_select] == 0) {
/* the minute button */
		t_collect_secs[t_select] = fl_get_timer(t_timer[t_select]);
#ifdef LINUX_CONSOLE
		if (linux_beep == 1) alarm_linux(4700,40);
#endif
		if (t_collect_secs[t_select] < MAXTIME) {
			t_collect_secs[t_select] += 60;
		}
		fl_set_timer(t_timer[t_select],t_collect_secs[t_select]);
		fl_suspend_timer(t_timer[t_select]);
		t_stopped[t_select] = 1;


	} else if (obj == t_quit) {
/* now handling of the quit button, wee need to check if any timer
   are still running and asking if one is shure, otherwise quit immediately
*/
		if (alarm_icon == 1 || t_is_running[0] == 1 ||
		    t_is_running[1] == 1 || t_is_running[2] == 1 ||
		    t_stopped[0] == 1 || t_stopped[1] == 1 || t_stopped[2] == 1)
		 {
		     if (fl_show_question("Timer still running/suspended/blinking\nReally quit?", 0))
		        exit(0);
		 } else {
		 	exit(0);
		 }
		        


	} /* end of t_quit */


     /* end of checking actions and keys */
	
  } /* end of while ################## end of the main loop */  
  ;

/* *********** we're finished, good bye **************** */
   return 0;
} /* end of main program */





