/* $Id: util.c,v 1.4 1999/03/05 14:29:09 bunka Exp $
 *
 *
 * Lab3Timer
 *
 * Copyright (C) 1999, Sebastian Bunka
 *
 * Distributed under the terms of the GNU GPL
 *
 *
 */

#include "forms.h"
#include "layout.h"
#include "help_window.h"
#include "msg_window.h"
#include "my_config.h"
#include "util.h"
#include "helptext.h"
#include <unistd.h>

/* 
   ##################################################
   and now the functions
   ##################################################
*/


void usage() {
	printf("%s",USAGE);
	exit(1);
} /* ende of usage() */



/* give a help screen */

void show_help_window() {
	char **p;
	extern char *helplines[];
/* now I want to have a fixed character width
 * so I use the function fl_set_browser_fontstyle;
 * help_ok = 1 prevents subsequent addition of the
 * textlines when the help window is opened more than 1 time
 */
	if (help_ok == 0) {
		fl_set_browser_fontsize(help_browser,FL_NORMAL_SIZE);
 		fl_set_browser_fontstyle(help_browser,FL_FIXED_STYLE);
   		for ( p = helplines; *p; p++) {
          		fl_add_browser_line(help_browser, *p);
			help_ok = 1;
		}
        }
          
	fl_show_form(help_window,FL_PLACE_CENTER,FL_TRANSIENT,"Lab3Timer - Help");
} /* end of show_help_window */



/* function to filter the output of the timer fields
 * will display the time string as 00:00:00
 * maybe one could write a much better one ;-))
 */
 
char *timer_filter(FL_OBJECT *ob, double secs) {

	int my_seconds = secs;
	int hour, minute, second;
	hour = my_seconds / 3600; /* get the FULL hours (hour = int!) */
	if (hour != 0) {
		my_seconds = my_seconds - (hour * 3600);
	}
	minute = my_seconds / 60;
	if (minute != 0) {
		my_seconds = my_seconds - (minute * 60);
	}
	second = my_seconds;

	sprintf(time_buf,"%.2d:%.2d:%.2d", hour, minute, second);
	return time_buf;

} /* end of timer_filter() */


/* this function formats the time elapsed since a timer has expired
 * if a timer is blinking, moving the mouse into the timer field
 * will display a short message with the time elapsed since the timer
 * has expired
 */
char *exp_filter(int my_timer, double secs) {

	int my_seconds = secs;
	int hour, minute, second;
	hour = my_seconds / 3600; /* get the FULL hours (hour = int!) */
	if (hour != 0) {
		my_seconds = my_seconds - (hour * 3600);
	}
	minute = my_seconds / 60;
	if (minute != 0) {
		my_seconds = my_seconds - (minute * 60);
	}
	second = my_seconds;
/* return the formatted string to the global buffer exp_buf */
	sprintf(exp_buf,"Timer %d has expired since\n%.2d:%.2d:%.2d", 
			my_timer + 1, hour, minute, second);
	return exp_buf;

} /* end of exp_filter() */




/* an alarm hack for linux from console_beep, blocks program display */

void alarm_linux(int freq, int length)
{
  FILE *dest;
  dest=fopen("/dev/console", "w");

  fprintf(dest,"\33[10;%d]\33[11;%d]\a\33[10]\33[11]", freq, length);
  fflush(dest);
  if (length) usleep(length*1000L); 

} /* end of alarm_linux() */
      



/* callbacks from form layout */
/* when a timer has expired */

void timer_expired(FL_OBJECT *ob, long data)
{
	char *text ;
	char *more_text = "Your comment was:";
	const char *last_text;
	int my_timer = 0;
	/* t_select 		global variable to pass the timer num to main */
	/* t_icon2, mask2  	global pixmap data def. in main */
	

#ifdef LINUX_CONSOLE
	int freq1=FREQ1, freq2=FREQ2, length=LENGTH, i;
#endif


/* ok, first find which timer number for the header alarm text */
	if ( data == 0 ) {
		text = "Timer ONE has expired";
		my_timer = 0;
	} else if (data == 1){
		text = "Timer TWO has expired";
		my_timer = 1;
	} else {
		text = "Timer THREE has expired";
		my_timer = 2;
	}

/* if this timer is already blinking == expired, don't repeat the alarm;
   sometimes xforms repeates an alerm twice immediately after the callback
   might be a bug.
 */
	if (timer_blinking[my_timer] == 1) return;


/* if no program is defined for use as alarm signal make the program beep */

	if ( !run_prog[0] ) {
	
#ifdef LINUX_CONSOLE
		if (linux_beep == 1){
			for (i=1; i<10; i++) {
				alarm_linux(freq1, length);
				alarm_linux(freq2, length);
			}
		} else {
			fl_ringbell(20);
		} /* endif linux_beep */
#else		
  	  fl_ringbell(20);	/* otherwise use the xforms function */
#endif
	} else {
		fl_exe_command(run_prog,0); 

	} /* end if !run_prog */


	last_text = fl_get_input(t_comment[my_timer]); /* text from comment */
  	fl_set_object_label(msg_text1,text);
  	fl_set_object_label(msg_text2,more_text);
  	fl_set_object_label(msg_text3,last_text);

/* we can switch off display of the alarm popup by cmd line option '-n' */
	if (no_popup == 0) {
		fl_show_form(msg_window,FL_PLACE_CENTER,FL_TRANSIENT,"Lab3Timer - Expired");
	}

	t_select = my_timer;      	/* give timer number  back to main */
	timer_blinking[my_timer] = 1;	/* indicator of which timer blinks */
	check_blink(1); 		/* checks if first timer expired and set icon/Title */

/* get the current sec, usecs of current time */
	fl_gettime(&exp_sec[my_timer],&exp_usec[my_timer]);  /* write the expiry time into array */
        fl_set_button(t_button[my_timer],1); 	/* set timers button for a quick clear */

   
return;

} /* end of timer_expired() */




/* post handler of t_timer will be called if mouse is in a timer field */
int post_timer(FL_OBJECT *ob, int ev, FL_Coord mx, FL_Coord y, int key, void *xev)
{
	int my_timer = 0;
	double exp_since = 0;
	long now_sec, now_usec;	/* for the current time */
	
/* mouse ENTER timer field */
	if (ev == FL_ENTER) {
		if (ob == t_timer[0]) my_timer = 0;
		if (ob == t_timer[1]) my_timer = 1;
		if (ob == t_timer[2]) my_timer = 2;

		if (timer_blinking[my_timer] == 1) {
			fl_gettime(&now_sec, &now_usec);
			exp_since = now_sec - exp_sec[my_timer]; /* calculate the time since timer has expired in secs */
			exp_filter(my_timer,exp_since);  /* exp_filter puts the formatted string in exp_buf */
			fl_show_oneliner(exp_buf,ob->form->x + ob->x,
		                   ob->form->y + ob->y + ob->h + 1);
		}

	} /* end ENTER */
   

/* mouse button push PUSH */
	if (ev == FL_PUSH) {
		if (ob == t_timer[0]) my_timer = 0;
		if (ob == t_timer[1]) my_timer = 1;
		if (ob == t_timer[2]) my_timer = 2;

		if (timer_blinking[my_timer] == 1) {
			timer_blinking[my_timer] = 0;
			check_blink(0);
		}
		fl_set_button(t_button[my_timer],1);
		t_select = my_timer;
		t_is_running[t_select] = 0;	/* for main to see that the timer isnt running or expired */
		fl_hide_oneliner();
	}


/* mouse LEAVE timer field */	
	if (ev == FL_LEAVE) {
		fl_hide_oneliner();
	}
	
  
return 0;
} /* end of post_timer */
  



int check_blink(int expired)	/* checks if any timer is expired and not cleared */
			/* and sets icon and title */
{

	if (timer_blinking[0] == 0 &&
	    timer_blinking[1] == 0 &&
	    timer_blinking[2] == 0) {
	    	fl_set_form_icon(layout, t_icon1, mask1);
	    	fl_set_form_title(layout,"Lab3Timer"); 
	    	alarm_icon = 0;		/* the only place where alarm_icon will get the 0 value */
	} else if (alarm_icon == 0 && expired == 1) {   
			/* is this the first alarm? then set icon and title */
		fl_set_form_icon(layout, t_icon2, mask2);
		fl_set_form_title(layout,"Lab3Timer - expired"); 
		alarm_icon = 1;
	}
	
	return alarm_icon;
}



